#!/usr/bin/env python3

import json
from enum import Enum

Type = Enum('Type', ['Grass', 'Poison', 'Fire', 'Flying', 'Water', 'Bug', 'Normal', 'Electric', 'Ground', 'Fairy',
                     'Fighting', 'Psychic', 'Rock', 'Steel', 'Ice', 'Ghost', 'Dragon']
            )

PRODUCTKEY = ""


def loadDex():
    with open("smalldex.json") as infile:
        dex = json.loads(infile.read())
        return dex


def loadProductKey():
    global PRODUCTKEY
    with open("productkey.txt") as infile:
        PRODUCTKEY = infile.read()


class TypeSet:
    def __init__(self, typelist):
        self.set = set(map(lambda x: Type[x], typelist))

    def __str__(self):
        return ", ".join(sorted(map(lambda x: x.name, self.set)))


def main():
    loadProductKey()
    dex = loadDex()
    try:
        while True:
            try:
                print("Search for a pokemon?")
                print("[1] By name")
                print("[2] By elemental type")
                print("[3] By name and elemental type")
                choice = int(input())
                opt_name, opt_type = {
                    1: (True, False),
                    2: (False, True),
                    3: (True, True)
                }[choice]

                pokname = ""
                poktypelist = [] 

                if opt_name:
                    print("Enter name: ", end="")
                    pokname = input()

                if opt_type:
                    print("Enter types to search (comma seperated): ")
                    poktypelist = input().split(',')
                
                poktypes = TypeSet(poktypelist)

                results = list(filter(lambda x: all(map(lambda y: y.name in x['type'], poktypes.set)), dex))
                results = list(filter(lambda x: x['name'].find(pokname) > -1, results))

                print()
                if len(results) > 0:
                    print("Results:")
                    for mon in results:
                        print("{ID:<4}{name}\t".format(**mon))
                    print()

                else:
                    if opt_name and not opt_type:
                        print("Could not find pokemon {0}".format(pokname))
                    elif not opt_name and opt_type:
                        print("Could not find pokemon of type {0}".format(poktypes))
                    else:
                        print("Could not find pokemon {0} of type {{0}}".format(pokname).format(poktypes))
            except (ValueError, KeyError):
                print("Invalid input")
    except (KeyboardInterrupt, EOFError):
        pass


if __name__ == "__main__":
    main()
